var generateGameContent = generateGameContent || {};

generateGameContent.generateGrid = function( columns , rows ){

	/*
	*	Going to draw a basic grid of Columns * Rows.
	*
	*/

	// geometry is a blank slate object class whereby the user can add vertices and faces manually allowing a custom shape.
	var geometry = new THREE.Geometry();

	// since we want to visually display a grid for our playing field we'll use the LineBasicMaterial in combination
	// with the LineSegments Object below.

	for( var i=0, x= -( columns * 0.05 ), z= -( rows * 0.05 ) , col=0 , row=0 , ii = rows * columns; i<ii; i++ ){

		// we want the grid to be centered, so if we want 20 rows with 20 columns then the starting 
		// x position will be -1 and the starting z position will be -1;

		/*
		* NOTE:
		* we're also going to use col and row to keep track of our grid construction. May seem redundent, however;
		* since we are multiplying x and z by 0.05 we will sometimes get float numbers that will screw up
		* our grid with things like 1.399999999 or 1.40000000001 if we rely on them to know when a particular
		* grid or row is finished.
		*/

		// each line segment needs to be made of a starting and ending vertice.
		// since we'll want to highlight an individual square when placing towers we'll want
		// to draw each square visible independently.

		// lines going from:

		// back to front
		geometry.vertices.push( new THREE.Vector3( x , 0 , z+0.1 ) , new THREE.Vector3( x , 0 , z ) );

		// left to right
		geometry.vertices.push( new THREE.Vector3( x , 0 , z ) , new THREE.Vector3( x+0.1 , 0 , z ) );

		x += 0.1, col++;

		if( col > columns-1 ){

			// we've drawn a complete row of half boxes, now we need to add a final line to complete the last box's edge
			// back to forward.
			geometry.vertices.push( new THREE.Vector3( x , 0 , z+0.1 ) , new THREE.Vector3( x , 0 , z ) );
			
			// reset x to the far left and increment z;
			z += 0.1, x = -( columns * 0.05 ), col = 0, row++;

			if( row > rows-1 ){

				x = -( columns * 0.05 );
				for( var j=0; j<columns; j++ ){

					// we've drawn all boxes except for the last row's final edges.
					geometry.vertices.push( new THREE.Vector3( x , 0 , z ) , new THREE.Vector3( x+0.1 , 0 , z ) );
					x+= 0.1;
				}
			}
		}
	}

	// we'll set the basic color of the grid to a soft gray.  
	// Adding the vertex color parameter will allow us to adjust the color of each vertice manually,
	// this will come in handy later when we want to show we've selected an individual square and what not.
	var lineMaterial = new THREE.LineBasicMaterial( { vertexColors: THREE.VertexColors } );

	// add one color for each vectice in our geometry.
	for( var i=0, ii=geometry.vertices.length; i<ii; i+=2 ){		
		geometry.colors[ i ] = new THREE.Color( 0.3 , 0.3 , 0.3 );
		geometry.colors[ i ].base = { r:0.3 , g:0.3 , b:0.3 };
    	geometry.colors[ i + 1 ] = geometry.colors[ i ];
    	geometry.colors[ i + 1 ].base = { r:0.3 , g:0.3 , b:0.3 };
	}

	/* TRY RANDOM COLORS
	// random color generation for the grid
	for( var i=0, ii=geometry.vertices.length; i<ii; i+=2 ){
		geometry.colors[ i ] = new THREE.Color( Math.random(), Math.random(), Math.random() );
    	geometry.colors[ i + 1 ] = geometry.colors[ i ];
	}
	//*/

	var grid = new THREE.LineSegments( geometry , lineMaterial );

	// Alternatively, if we were to use the THREE.Line() object class lines would be drawn sequentially from vertice to vertice
	/* TRY EXAMPLE 
	grid = new THREE.Line( geometry, lineMaterial );
	//*/

	// we're going to add some custom values to the grid so that later on we can retrieve them
	grid.columns = columns;
	grid.rows = rows;

	return grid;
}

generateGameContent.generateGridTargetMesh = function( grid ){

	// since we already generated the vertices for grid, we'll use that vectorr list to copy
	// vectors to our target mesh.
	var v = 0; // used as a vertices counter
	var faceOne, faceTwo;
	var vectorList = grid.geometry.vertices;

	var geometry = new THREE.Geometry();
	var basicMaterial = new THREE.MeshLambertMaterial( { color: 0x0000ff , visible: false , side: THREE.DoubleSide });
	/* VIEW MESH
	basicMaterial.visible = true;
	//*/

	for( var i=0, ii=vectorList.length; i<ii; i++ ){
		geometry.vertices.push( vectorList[i].clone() );
	}

	// now we'll create 2 faces for each square, but we'll need to know how many rows and columns there are.
	for( var i=0, ii=grid.rows, f=0/* f is used as a local face counter */; i<ii; i++ ){
		for( var j=0, jj=grid.columns; j<jj; j++ ){

			// reference the vertices index position to draw the faces.
			faceOne = new THREE.Face3( v , v+3 , v+1 );
			faceOne.index = f;

			faceTwo = new THREE.Face3( v , v+4 , v+3 );
			faceTwo.index = f;

			geometry.faces.push( faceOne , faceTwo );
			f++;
			v+=4;
		}
		v+=2;

	}

	// in order for the targeting to work later on we'll need to do the following calculations
	geometry.computeFaceNormals();
	geometry.computeBoundingSphere();

	var gridTargetMesh = new THREE.Mesh( geometry , basicMaterial );

	return gridTargetMesh;
}

generateGameContent.createCells = function( grid , gridTargetMesh ){

	var cells = [];

	// lineVertices: for each line drawing each side of each visual cell there are two vertices.  
	// 				 one for the start and end points.
	// surroundingCells: Indexed position of all surrounding cells in a clockwise fashion starting to the left.
	// 					 each number represents the index location within surroundingCells that contains the index 
	// 					 location of the cell within cell.list[].
	//
	//    | 1 | 2 | 3 |		----> example 		| 43 | 44 | 45 |
	//    | 0 | x | 4 |							| 60 | xx | 62 |
	// 	  | 7 | 6 | 5 |							| 79 | 78 | 77 |	----> will appear in the array as follows . . . 
	// 
	//	  . . [ 60 , 43 , 44 , 45 , 62 , 77 , 78 , 79 ]
	//
	// travelTo: is the passage from this cell to the indexed cell in surroundingCells allowed?
	// currentIteration: at the beginning of the pathfinding function this array is set to equal travelTo for each given cell
	// 					 then it is updated through each iteration for each cell and travelTo is never adjusted.

	// we're going to establish cell index/ID -1.  This will act as a stop for later logic.
	cells[ -1 ] = {
		lineVertices: [],
		surroundingCells: [ -1,-1,-1,-1,-1,-1,-1,-1 ],
		travelTo: [ false, false, false, false, false, false, false, false ],
		currentIteration: [ false, false, false, false, false, false, false, false ],
	}

	var vectorList = grid.geometry.vertices;
	var v = 0; // used as vertices counter;
	var c = 0; // used as cell counter;

	// we'll need to know how many rows and columns there are.

	// the first cell of each row will be a goal cell.  If the enemy make it to a goal square it will have escaped.

	// create first cell of first row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
		surroundingCells: [ -1,-1,-1,-1, 1 , grid.columns+1 , grid.columns , -1 ],
		travelTo: [ false, false, false, false, true, true, true, false ],
		currentIteration: [ false, false, false, false, true, true, true, false ],
		towerID: -1,
		goal: true,
		start: false,
		first: true,
		last: false,
	}
	v+=4;
	c++;

	//generate first row minus first and last cell
	for( var j=1, jj=grid.columns-1; j<jj; j++ ){

		// create object for each new cell with the following:
		// - index position of vertices needed to draw line segements.
		// - index position of surrounding cells.

		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
			surroundingCells: [ c-1 , -1,-1,-1, c+1 , c + grid.columns + 1 , c + grid.columns , c + grid.columns - 1 ],
			travelTo: [ true, false, false, false, true, true, true, true ],
			currentIteration: [ true, false, false, false, true, true, true, true ],
			towerID: -1,
			goal: false,
			start: false,
		}
		v+=4;
		c++;
	}

	// create last cell of first row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
		surroundingCells: [ c-1, -1,-1,-1,-1,-1 , c + grid.columns , c + grid.columns-1 ],
		travelTo: [ true, false, false, false, false, false, true, true ],
		currentIteration: [ true, false, false, false, false, false, true, true ],
		towerID: -1,
		goal: false,
		start: true,
		first: true,
		last: false,
	}
	v+=6;
	c++;

	for( var i=1, ii=grid.rows-1; i<ii; i++ ){

		// create first cell of row
		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
			surroundingCells: [ -1,-1,
				c - grid.columns ,	
				c - ( grid.columns - 1 ),
				c + 1 , 
				c + ( grid.columns + 1 ) , 
				c + grid.columns , -1				
			],
			travelTo: [ false, false, true, true, true, true, true, false ],
			currentIteration: [ false, false, true, true, true, true, true, false ],
			towerID: -1,
			goal: true,
			start: false,
			first: false,
			last: false,
		}
		v+=4;
		c++;

		for( var j=1, jj=grid.columns-1; j<jj; j++ ){

			// create object for each new cell with the following:
			// - index position of vertices needed to draw line segements.
			// - index position of surrounding cells.

			cells[c] = {
				lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
				surroundingCells: [ 
					c - 1 ,
					c - ( grid.columns + 1 ),
					c - grid.columns ,
					c - ( grid.columns - 1 ),
					c + 1 ,
					c + ( grid.columns + 1 ) , 
					c + grid.columns , 
					c + ( grid.columns - 1 )					
				],
				travelTo: [ true, true, true, true, true, true, true, true ],
				currentIteration: [ true, true, true, true, true, true, true, true ],
				towerID: -1,
				goal: false,
				start: false,
			}
			v+=4;
			c++;
		}

		// create last cell of row
		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 , v+( grid.columns*4 )+4 , v+( grid.columns*4 )+5 ],
			surroundingCells: [ 
				c - 1 ,
				c - ( grid.columns + 1 ),
				c - grid.columns , -1,-1,-1,
				c + grid.columns , 
				c + ( grid.columns - 1 ) , 							
			],
			travelTo: [ true, true, true, false, false, false, true, true ],
			currentIteration: [ true, true, true, false, false, false, true, true ],
			towerID: -1,
			goal: false,
			start: true,
			first: false,
			last: false,
		}
		v+=6;
		c++;
	}

	//create first cell of last row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 ],
		surroundingCells: [ -1,-1, c-grid.columns , c - ( grid.columns-1 ) , c+1, -1,-1,-1 ],
		travelTo: [ false, false, true, true, true, false, false, false ],
		currentIteration: [ false, false, true, true, true, false, false, false ],
		towerID: -1,
		goal: true,
		start: false,
		first: false,
		last: true,
	}
	v+=4;
	c++;

	//generate last row minus first and last cell
	for( var j=1, jj=grid.columns-1; j<jj; j++ ){

		// create object for each new cell with the following:
		// - index position of vertices needed to draw line segements.
		// - index position of surrounding cells.

		cells[c] = {
			lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 ],
			surroundingCells: [  
				c - 1 ,
				c - ( grid.columns + 1 ),
				c - grid.columns ,
				c - ( grid.columns - 1 ),
				c + 1 , -1,-1,-1,
			],
			travelTo: [ true, true, true, true, true, false, false, false ],
			currentIteration: [ true, true, true, true, true, false, false, false ],
			towerID: -1,
			goal: false,
			start: false,
		}
		v+=4;
		c++;
	}

	// create last cell of last row
	cells[c] = {
		lineVertices: [ v , v+1 , v+2 , v+3 , v+4 , v+5 ],
		surroundingCells: [ c -1 , c - ( grid.columns + 1 ) , c - grid.columns , -1,-1,-1,-1,-1 ],
		travelTo: [ true, true, true, false, false, false, false, false ],
		currentIteration: [ true, true, true, false, false, false, false, false ],
		towerID: -1,
		goal: false,
		start: true,
		first: false,
		last: true,
	}
	
	// add remaining vertices to last row
	c -= ( grid.columns-1 )
	v =  grid.geometry.vertices.length - ( grid.columns*2 );
	for( var i=0, ii=grid.columns; i<ii; i++ ){
		cells[c].lineVertices.push( v , v+1 );
		v+=2;
		c++;
	}

	// we're going to create a list of all those cells which act as a starting location.
	// then when an enemy spawns, it will randomly spawn on a starting cell.
	var startingIndexes = [];
	for( var i=0, ii=cells.length; i<ii; i++ ){
		if( cells[ i ].start ){
			startingIndexes.push( i );
		}
	}

	// we're going to need to know the center position of the cell.  For that we'll cycle through all
	// of the gridTargetMesh faces and average out all the vertices for each face.  Then we'll record that to each cell.
	var verticeArray = [];
	var faceHolder = gridTargetMesh.geometry.faces;
	var verticesHolder = gridTargetMesh.geometry.vertices;
	
	// first we'll construct an array full of zeroed out vertices equal in length to the length of cells.
	for( var i=0, ii=cells.length; i<ii; i++ ){
		verticeArray.push( new THREE.Vector3( 0 , 0 , 0 ) );
	}
	// now we'll cycle though all faces, we'll use the index position of the face to target our vertices array.
	for( var i=0, ii=faceHolder.length; i<ii; i++ ){
		verticeArray[ faceHolder[ i ].index ].add( verticesHolder[ faceHolder[ i ].a ].clone() );
		verticeArray[ faceHolder[ i ].index ].add( verticesHolder[ faceHolder[ i ].b ].clone() );
		verticeArray[ faceHolder[ i ].index ].add( verticesHolder[ faceHolder[ i ].c ].clone() );
	}
	// lastely, we'll divide each vertice by 6 and then add it to cells.
	for( var i=0, ii=cells.length; i<ii; i++ ){
		verticeArray[ i ].divideScalar( 6 );
		cells[ i ].center = verticeArray[ i ].clone();
	}

	// we're going to update the base color of the grid so that the first and last rows appear darker
	// this will indicate the starting area and ending area.
	for( var i=0, ii=cells.length; i<ii; i++ ){
		if( cells[ i ].start && !cells[ i ].first && !cells[ i ].last ){
			grid.geometry.colors[ cells[ i ].lineVertices[ 2 ] ].base = { r:0.1 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 3 ] ].base = { r:0.1 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 6 ] ].base = { r:0.1 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 7 ] ].base = { r:0.1 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 2 ] ].setRGB( 0.1 , 0.1 , 0.3 );
			grid.geometry.colors[ cells[ i ].lineVertices[ 3 ] ].setRGB( 0.1 , 0.1 , 0.3 );
			grid.geometry.colors[ cells[ i ].lineVertices[ 6 ] ].setRGB( 0.1 , 0.1 , 0.3 );
			grid.geometry.colors[ cells[ i ].lineVertices[ 7 ] ].setRGB( 0.1 , 0.1 , 0.3 );
			delete cells[ i ].first;
			delete cells[ i ].last;
		} else if( cells[ i ].goal && !cells[ i ].first && !cells[ i ].last ) {
			grid.geometry.colors[ cells[ i ].lineVertices[ 2 ] ].base = { r:0.3 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 3 ] ].base = { r:0.3 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 6 ] ].base = { r:0.3 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 7 ] ].base = { r:0.3 , g:0.1 , b: 0.3 };
			grid.geometry.colors[ cells[ i ].lineVertices[ 2 ] ].setRGB( 0.3 , 0.1 , 0.3 );
			grid.geometry.colors[ cells[ i ].lineVertices[ 3 ] ].setRGB( 0.3 , 0.1 , 0.3 );
			grid.geometry.colors[ cells[ i ].lineVertices[ 6 ] ].setRGB( 0.3 , 0.1 , 0.3 );
			grid.geometry.colors[ cells[ i ].lineVertices[ 7 ] ].setRGB( 0.3 , 0.1 , 0.3 );
			delete cells[ i ].first;
			delete cells[ i ].last;
		}
	}

	grid.geometry.colorsNeedUpdate = true;

	return { list:cells , previousSelection:[] , info:[] , startingIndexes:startingIndexes };
}

//////////////////////////////////////////////////////////////

generateGameContent.createGameParameters = function(){

	// these paraments will be used throughout

	var gameParameters = {

		gridRows: 7,	
		gridColumns: 11, 
		totalWalls: 24,
		towerWalls: 24,
		tower: {
			tetra: { // hotkey 1 , 49
				allowed: 3, // only 3 tetra towers can be built.
				built: 0,
			},
			cube: { // hotkey 2 , 50
				allowed: 1, // only 1 cube tower can be built.
				built: 0,
			},
			hex: { // hotkey 3 , 51
				allowed: 2, // only 2 hellfire hex towers are allowed.
				built: 0,
			},
			tack: { // hotkey 4 , 52
				allowed: 1, // only 1 tack tower is allowed.
				built: 0,
			},
			previousSelection: 'tetra', // will be used to cancel previously selected tower type.
			modelBase: models.tetraBase(), // text holding the model information to build a Mesh.
			modelTurret: models.tetraTurret(), // text holding the model information to build a Mesh.
		},

		// DEBUG PARAMETERS
		startWaveManager: false,

	}

	return gameParameters;
}

generateGameContent.towerDetails = function( value ){

	var object;
	var basicMaterial = new THREE.MeshLambertMaterial( { vertexColors: THREE.FaceColors } );

	switch( value ){

		case 'tetra':

			object = {
				fireSchedule: [ 51 , 45 , 39 , 33 , 27 , 21 , 15 ], // ms before the tower fires again.  Each entry is for a different upgrade level
				targetRadius: [ 0.15 , 0.155 , 0.16 , 0.165 , 0.17 , 0.175 , 0.18 ], // the radius the tower has effect.
				rest: 0, // rest increments every animation frame.  Once it equals the fireRate entry it will cause the tower to fire.
				barrelToFire: 0, // starts at 0, then increments.  resets once it reaches barrels index length.
				level: 0, // tower levels start at 0.  this will indicate the rireRate index to follow.
				animating: false, // if set to true the lookAt function will prevent from firing, it will indicate the tower fire animation is in progress.
				animation: [ // animations are 3D models used during this towers animation cycle.  Each tower will have different animations.
					new THREE.Mesh( models.import( models.tetraFireOne() , 10 ) , basicMaterial ) ,
					new THREE.Mesh( models.import( models.tetraFireTwo() , 10 ) , basicMaterial )
				],
				targetShape: -1, // if this value is populated then that means this tower has a dedicated target shape.
			};
			// we'll scale the animiations for this tower
			for( var i=0, ii=object.animation.length; i<ii; i++ ){
				object.animation[ i ].scale.set( 0.015 , 0.015 , 0.015 + ( i * 0.005 ) );
				object.animation[ i ].visible = false;
				scene.add( object.animation[ i ] );
			}
			break;

		case 'cube':

			object = {
				fireSchedule: [ 150 , 130 , 110 , 90 ], // ms before the tower fires again.  Each entry is for a different upgrade level
				targetRadius: [ 0.16 , 0.17 , 0.18 , 0.19 ], // the radius the tower has effect.
				rest: 0, // rest increments every animation frame.  Once it equals the fireRate entry it will cause the tower to fire.
				level: 0, // tower levels start at 0.  this will indicate the rireRate index to follow.
				animating: false, // if set to true the lookAt function will prevent from firing, it will indicate the tower fire animation is in progress.
				animation: [ // animations are 3D models used during this towers animation cycle.  Each tower will have different animations.
					new THREE.Mesh( models.import( models.cubeFireOne() , 10 ) , basicMaterial ) ,
					new THREE.Mesh( models.import( models.cubeFireTwo() , 10 ) , basicMaterial ) , 
					new THREE.Mesh( models.import( models.cubeFireThree() , 10 ) , basicMaterial ) ,
					//new THREE.Mesh( models.import( models.cubeFireTwo() , 10 ) , basicMaterial )
				],
				targetShape: -1, // if this value is populated then that means this tower has a dedicated target shape.
			};
			// we'll scale the animiations for this tower
			for( var i=0, ii=object.animation.length; i<ii; i++ ){
				object.animation[ i ].scale.set( 0.01 , 0.01 , 0.01 );
				object.animation[ i ].visible = false;
				scene.add( object.animation[ i ] );
			}

			break;

	}	

	return object;
}

generateGameContent.createUserSelectionTaskManager = function(){

	// within the task manager we'll keep track of various 
	// user selected tasks such as placing towers, upgrading,
	// menu selections and so on.

	// Since we're going to add a graphical user interface at 
	// a later time we're going to allow for functionality 
	// through hot keys.  

	var userTaskManager = {

		// if a user presses the same button twice then it will cancel the selection.

		createWalls: false, // hotkey w , 87
		createTowers: false, // hotkey t , 84
		
	}

	return userTaskManager;
}

generateGameContent.generateWallMesh = function(){

	var geometry = new THREE.Geometry();

	var wallMaterial = new THREE.MeshLambertMaterial( { color: 0x8888bb /*, side: THREE.DoubleSide*/ } );

	for( var i=0, ii=gameParameters.totalWalls*4; i<ii; i+=4 ){

		// add four vertices for each corner of a wall
		geometry.vertices.push( 	new THREE.Vector3( 0 , 0 , 0 ) ,
								new THREE.Vector3( 0 , 0 , 0 ) , 
								new THREE.Vector3( 0 , 0 , 0 ) ,
								new THREE.Vector3( 0 , 0 , 0 ) );

		// add two faces for the wall, since the material is double sided we only need two
		geometry.faces.push( new THREE.Face3( i+1 , i , i+2 ) , new THREE.Face3( i+1 , i+2 , i+3 ) );

	}

	var walls = new THREE.Mesh( geometry , wallMaterial );

	// we're going to add a custom array to walls.
	// it will serve as a look up array to see if a wall already exists at a given vertice.
	walls.lookUpArray = [];

	// we'll also create an info Array.  That way the passage is preserved if surrounding things are removed.
	walls.infoArray = [];  

	// a bufferIndex to let us know what current index position to use for newly placed walls
	walls.bufferIndex = { vertices: 0 , maxVertices: ( gameParameters.totalWalls*4 ) };

	// a link array telling us which line segement is associated with which buffer index position
	// will come in handy when removing walls.
	walls.bufferLink = [];

	// an available index array incase the user wants to remove a wall segment.
	// will come in handy when removing walls.
	walls.availableIndexes = [];

	return walls;	
}

generateGameContent.generateTowerWallMesh = function(){

	var geometry = new THREE.Geometry();

	var towerWallMaterial = new THREE.MeshLambertMaterial( { color: 0x8888bb /*, side: THREE.DoubleSide*/ } );

	for( var i=0, ii=gameParameters.towerWalls*4; i<ii; i+=4 ){

		// add four vertices for each corner of a wall
		geometry.vertices.push( 	new THREE.Vector3( 0 , 0 , 0 ) ,
								new THREE.Vector3( 0 , 0 , 0 ) , 
								new THREE.Vector3( 0 , 0 , 0 ) ,
								new THREE.Vector3( 0 , 0 , 0 ) );

		// add two faces for the wall, since the material is double sided we only need two
		geometry.faces.push( new THREE.Face3( i+1 , i , i+2 ) , new THREE.Face3( i+1 , i+2 , i+3 ) );

	}

	var towerWalls = new THREE.Mesh( geometry , towerWallMaterial );

	// a bufferIndex to let us know what current index position to use for newly placed walls
	towerWalls.bufferIndex = { vertices: 0 , maxVertices: ( gameParameters.towerWalls*4 ) };

	// an available index array incase the user wants to remove a wall segment.
	// will come in handy when removing towers.
	towerWalls.availableIndexes = [];

	return towerWalls;	
}

//////////////////////////////////////////////////////////////

generateGameContent.createColorTemplate = function(){

	var array = [

		// these values are manually retrieved from model information.
		// changing the colors of the enemies will require these values to be 
		// updated to reflect the changes.
		// all values are multiplied by 3 because pentagons are made up of 3 faces.
		// but there actual in game value is 1/3 of the value listed below.
		{ r:1 , g:0.4235294117647059 , b:0 , value:3 , type:'triangles' } ,
		{ r:0 , g:0.5333333333333333 , b:1 , value:15 , type:'squares' } ,
		{ r:0.9803921568627451 , g:0.4470588235294118 , b:0.4980392156862745 , value:1 , type:'pentagons' },
	];

	return array;
}

generateGameContent.createStages = function(){

	// this game will feature 5 stages.
	// each stage will have a number of waves

	var stages = [];

	// each stage will have an arrays representing waves.
	// each pair of elements within a wave will represent the index position of the shape in the shapeLexicon.
	// the second element of the pair represents the delay before firing the next index position.
	var stageOne = [
	
		[ 0 , 1 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 ],
		[ 0 , 240 , 0 , 40 , 0 , 40 , 0 , 40 , 0 , 40 ],
		[ 0 , 240 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 , 0 , 60 ],
		[ 2 , 240 , 0 , 60 , 0 , 60 , 2 , 60 , 0 , 60 , 0 , 60 ],
		[ 2 , 240 , 0 , 40 , 0 , 40 , 2 , 40 , 0 , 40 , 0 , 40 ],
		[ 2 , 240 , 0 , 60 , 0 , 60 , 2 , 60 , 0 , 60 , 0 , 60 , 2 , 60 , 0 , 60 , 0 , 60 , 2 , 60 ],
		[ 4 , 240 , 2 , 60 , 0 , 60 , 4 , 60 , 2 , 60 , 0 , 60 ],
		[ 4 , 240 , 2 , 40 , 0 , 40 , 4 , 40 , 2 , 40 , 0 , 40 ],
		[ 4 , 240 , 2 , 60 , 0 , 60 , 4 , 60 , 2 , 60 , 0 , 60 , 4 , 60 , 2 , 60 , 0 , 60 , 4 , 60 ],
		[ 2 , 240 , 2 , 60 , 2 , 60 , 4 , 60 , 4 , 60 , 4 , 60 , 6 , 1 , 6 , 1 , 6 , 1 ],		
		
	];

	// add stages to stages array
	stages.push( stageOne );

	return stages;
}

generateGameContent.loadStage = function( stage , shapesToSpawn ){

	var model, lines;

	// each enemy that we load into the stage will be assigned with a uniqueID
	var uniqueID = 0;

	var basicMaterial = new THREE.MeshLambertMaterial( { vertexColors: THREE.FaceColors , side: THREE.DoubleSide });
	var basicLineMaterial = new THREE.LineBasicMaterial( { vertexColors: THREE.FaceColors , linewidth: 2 } );

	// cycle through all the waves to the stage in question.
	for( var wave=0, waves=stageManager[ stage ].length; wave<waves; wave++ ){

		// cycle through all elements of the array.
		// we're going to skip over every second element because the second element represents
		// the delay before the shape is added to the game session.
		for( var i=0, ii=stageManager[ stage ][ wave ].length; i<ii; i+=2 ){

			// retrieve the model to clone
			model = new THREE.Mesh( models.import( shapeLexicon[ stageManager[ stage ][ wave ][ i ] ] , 10 ) , basicMaterial ); ;
			lines = new THREE.LineSegments( models.importLine( shapeLexicon[ stageManager[ stage ][ wave ][ i ]+1 ] , 9.8 ) , basicLineMaterial ); ;

		/*
			// REMOVE BELOW
			if( stageManager[ stage ][ wave ][ i ] == 18 ){

				var f;
				var v, V, d, D1, D2;
				for( var i=0, ii=lines.geometry.vertices.length; i<ii; i+=2 ){

					v = new THREE.Vector3();

					v = lines.geometry.vertices[ i ].clone();
					V = lines.geometry.vertices[ i+1 ].clone();

					f = new THREE.Line3( v.clone() , V.clone() );
					d = v.distanceTo( V );

					D1 = 6.386;
					D2 = d - 6.386;

					D1 = D1 / d;
					D2 = D2 / d;

					v = f.at( D1 );
					V = f.at( D2 );

					console.log( '{"x":' + v.x + ',"y":' + v.y + ',"z":' + v.z + '},');
					console.log( '{"x":' + V.x + ',"y":' + V.y + ',"z":' + V.z + '},');
				}
			}
		*/

			// copy over information relative to this game.
			model = enemyDetails( model );

			model.scale.set( 0.004 , 0.004, 0.004 );
			model.position.set( 0 , 0 , 0 );
			model.visible = false;
			model.uniqueID = uniqueID;
			
			// calculate break animation offsets
			model.break=[];

			// these vertice offsets will be applied to each linesegment.
			for( var j=0, jj=lines.geometry.vertices.length*0.5; j<jj; j++ ){
				model.break[ j ] = new THREE.Vector3( randomNumber() , randomNumber() , randomNumber() );
				model.break[ j ].normalize();
			}

			// increment uniqueID.
			uniqueID++;

			lines.scale.set( 0.0041 , 0.0041 , 0.0041 );
			lines.position.set( 0 , 0 , 0 );
			lines.visible = false;

			// copy over the delay before this enemy spawns
			model.timeToSpawn = stageManager[ stage ][ wave ][ i+1 ];

			// add the clone to shapesToSpawn
			shapesToSpawn.push( model );
			shapesToSpawn.push( lines );
		}
	}

	// increment stage
	stage ++;

}

//////////////////////////////////////////////////////////////

function enemyDetails( m ){

	// each enemy will have an object representing the number of shape types it has
	m.shapes = {
		triangles:0,
		squares:0,
		pentagons:0,
		hexagons:0,
		total:0,
	}

	// now we want to collect the index positions of the different colored faces.
	// that way we know which face to 'break' once a hit is registered.
	m.shapeIndex = {
		triangles:[], // this array will hold the indexed position of all associated faces.
		triangleIndex:0, // this value will tell us which position within the above array to target.
		squares:[],
		squareIndex:0,
		pentagons:[], // when a face is destroyed it will be transfered to pentagonsToRegnerate array.
		pentagonsToRegernerate:[], // when a face is restored, it will be transfered back to pentagons array
		countDown: 0, // when the countDown reaches 60 it will trigger a regeneration effect.
		regenerate: false, // if this is set to true it will trigger the countDown
		hexagons:[],
		hexagonIndex:0,
	}

	var c;

	for( var i=0, ii=m.geometry.faces.length; i<ii; i++ ){
		
		c = m.geometry.faces[ i ].color;
		for( var j=0, jj=colorTemplate.length; j<jj; j++ ){
			if( c.r == colorTemplate[ j ].r && c.g == colorTemplate[ j ].g && c.b == colorTemplate[ j ].b ){

				// record how many of a certain shape their are and increment the value associated.
				m.shapes[ colorTemplate[ j ].type ] += colorTemplate[ j ].value;
				m.shapes.total += colorTemplate[ j ].value;

				// add this indexed face position to the shapeIndex array.
				m.shapeIndex[ colorTemplate[ j ].type ].push( i );
			}
		}
	}

	// load the breaking sound for this object
	m.soundFX = new Audio("sounds/brokenOne.wav");
	m.broken = false;
	m.spawn = true;
	m.path = [];

	return m;
}

function randomNumber(){

	if( Math.random() > 0.5 ){
		return Math.random();
	} else {
		return -Math.random();
	}

}